package nl.andrewlalis.gymboard_api.model.auth;

import jakarta.persistence.*;
import org.hibernate.annotations.CreationTimestamp;

import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;

@Entity
@Table(name = "auth_user")
public class User {
	@Id
	@Column(nullable = false, updatable = false, length = 26)
	private String id;

	@CreationTimestamp
	private LocalDateTime createdAt;

	@Column(nullable = false)
	private boolean activated;

	@Column(nullable = false, unique = true)
	private String email;

	@Column(nullable = false)
	private String passwordHash;

	@Column(nullable = false)
	private String name;

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(
			name = "auth_user_roles",
			joinColumns = @JoinColumn(name = "user_id"),
			inverseJoinColumns = @JoinColumn(name = "role_short_name")
	)
	private Set<Role> roles;

	public User() {}

	public User(String id, boolean activated, String email, String passwordHash, String name) {
		this.id = id;
		this.activated = activated;
		this.email = email;
		this.passwordHash = passwordHash;
		this.name = name;
		this.roles = new HashSet<>();
	}

	public String getId() {
		return id;
	}

	public LocalDateTime getCreatedAt() {
		return createdAt;
	}

	public boolean isActivated() {
		return activated;
	}

	public String getEmail() {
		return email;
	}

	public String getPasswordHash() {
		return passwordHash;
	}

	public String getName() {
		return name;
	}

	public Set<Role> getRoles() {
		return roles;
	}
}
